import { DataSource, ViewColumn, ViewEntity } from "typeorm";
import { memberExecutivePositions } from "../entity/club/member/memberExecutivePositions";
import { DB_TYPE } from "../env.defaults";

let durationInDays: string;
let durationInYears: string;
let exactDuration: string;
if (DB_TYPE == "postgres") {
  durationInDays = `SUM(COALESCE("memberExecutivePositions"."end", CURRENT_DATE) - "memberExecutivePositions"."start")`;
  durationInYears = `SUM(EXTRACT(YEAR FROM CURRENT_DATE) - EXTRACT(YEAR FROM member.birthdate))`;
  exactDuration = `SUM(AGE(COALESCE("memberExecutivePositions"."end", CURRENT_DATE), "memberExecutivePositions"."start"))`;
} else if (DB_TYPE == "mysql") {
  durationInDays = `SUM(DATEDIFF(COALESCE(memberExecutivePositions.end, CURDATE()), memberExecutivePositions.start))`;
  durationInYears = `SUM(TIMESTAMPDIFF(YEAR, memberExecutivePositions.start, COALESCE(memberExecutivePositions.end, CURDATE())))`;
  exactDuration = `
    CONCAT(
      SUM(FLOOR(TIMESTAMPDIFF(DAY, memberExecutivePositions.start, COALESCE(memberExecutivePositions.end, CURDATE())) / 365.25)),
      ' years ',
      SUM(FLOOR(MOD(TIMESTAMPDIFF(MONTH, memberExecutivePositions.start, COALESCE(memberExecutivePositions.end, CURDATE())), 12))),
      ' months ',
      SUM(FLOOR(MOD(TIMESTAMPDIFF(DAY, memberExecutivePositions.start, COALESCE(memberExecutivePositions.end, CURDATE())), 30))),
      ' days'
    )
  `;
} else if (DB_TYPE == "sqlite") {
  durationInDays = `SUM(JULIANDAY(COALESCE(memberExecutivePositions.end, DATE('now'))) - JULIANDAY(memberExecutivePositions.start))`;
  durationInYears = `SUM(FLOOR((JULIANDAY(COALESCE(memberExecutivePositions.end, DATE('now'))) - JULIANDAY(memberExecutivePositions.start)) / 365.25))`;
  exactDuration = `
    SUM((strftime('%Y', COALESCE(memberExecutivePositions.end, DATE('now'))) - strftime('%Y', memberExecutivePositions.start))) || ' years ' ||
      SUM((strftime('%m', COALESCE(memberExecutivePositions.end, DATE('now'))) - strftime('%m', memberExecutivePositions.start))) || ' months ' ||
      SUM((strftime('%d', COALESCE(memberExecutivePositions.end, DATE('now'))) - strftime('%d', memberExecutivePositions.start))) || ' days'
  `;
}

@ViewEntity({
  expression: (datasource: DataSource) =>
    datasource
      .getRepository(memberExecutivePositions)
      .createQueryBuilder("memberExecutivePositions")
      .select("executivePosition.id", "positionId")
      .addSelect("executivePosition.position", "position")
      .addSelect("member.id", "memberId")
      .addSelect("member.firstname", "memberFirstname")
      .addSelect("member.lastname", "memberLastname")
      .addSelect("member.nameaffix", "memberNameaffix")
      .addSelect("member.birthdate", "memberBirthdate")
      .addSelect("salutation.salutation", "memberSalutation")
      .addSelect(durationInDays, "durationInDays")
      .addSelect(durationInYears, "durationInYears")
      .addSelect(exactDuration, "exactDuration")
      .leftJoin("memberExecutivePositions.executivePosition", "executivePosition")
      .leftJoin("memberExecutivePositions.member", "member")
      .leftJoin("member.salutation", "salutation")
      .groupBy("executivePosition.id")
      .addGroupBy("member.id")
      .addGroupBy("salutation.id"),
})
export class memberExecutivePositionsView {
  @ViewColumn()
  durationInDays: number;

  @ViewColumn()
  durationInYears: number;

  @ViewColumn()
  exactDuration: string;

  @ViewColumn()
  position: string;

  @ViewColumn()
  positionId: number;

  @ViewColumn()
  memberId: string;

  @ViewColumn()
  memberSalutation: string;

  @ViewColumn()
  memberFirstname: string;

  @ViewColumn()
  memberLastname: string;

  @ViewColumn()
  memberNameaffix: string;

  @ViewColumn()
  memberBirthdate: Date;
}
